/****************************************************************************
* (C) 1999-2005 by BECK IPC GmbH
*
*  BECK IPC GmbH
*  Germany
*
*  http://www.beck-ipc.com
*
* ---------------------------------------------------------------------------
* Module         : HELPER.C
* Function       : Some useful helper functions
* ---------------------------------------------------------------------------

$Header: c:\prjPib\IPC@CHIP\CLib\Helper\HELPER.C, 18, 17.11.2005 15:30:04, Andre Pribil$

*****************************************************************************/

#include <stdarg.h>

#include "config.h"
#include "common.h"
#include "helper.h"

/*
 @doc DOC_INDEX
 ---------------------------------------------------------------
 Helper API Table of Contents
 ---------------------------------------------------------------
 @clibidx  <tab><cfont>#include <InBraces HELPER\.H><efont> |

 @libcat1x  | Character output | helper01 |
    helper_strout | String output |
 @libcat1x  | Character output | helper02 |
    helper_chrout | Char output |
 @libcat1x  | Character output | helper03 |
    helper_dezout | Dezimal output |
 @libcat1x  | Character output | helper04 |
    helper_hexdig | Hex digit output |
 @libcat1x  | Character output | helper05 |
    helper_hexout | Hex value output |
 @libcat1x  | Character output | helper06A |
    helper_printf | Printf output |
 @libcat1x  | Character output | helper06 |
    helper_sprintf | String formatting |

 @libcat2x  |  Miscellaneous  | helper07 |
    helper_alloc_rtos_mem | Allocate RTOS memory |
 @libcat2x  |  Miscellaneous  | helper08 |
    helper_release_rtos_mem | Release allocated RTOS memory |
 @libcat2x  |  Miscellaneous  | helper09 |
    helper_alloc_rtos_fast_mem | Allocate RTOS memory from fast memory pool|
*/


#ifdef _helper01_
/*
 @doc DOC_helper01_
 ---------------------------------------------------------------
 Strout
 ---------------------------------------------------------------
 @clib  helper_strout | String output function |
        <cfont>void helper_strout ( const char far *str ); <efont> |
 @parm  char far | str  | Zero terminated output string to print over STDOUT
 @rdesc -- none --
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../dosemu.htm#0x210x09<linkmid>documentation<linktail>
        for more details.
 @target     SC12  | SC13  | SC11  | SC1x3
 @version    V1.10 | V1.00 | V1.00 | V0.90
*/
void helper_strout( const char far *s )
{
  asm push ds
  asm lds  dx, s
  asm mov  ah, 0x9
  asm int  0x21
  asm pop  ds
} // helper_strout
#endif


#ifdef _helper02_
/*
 @doc DOC_helper02_
 ---------------------------------------------------------------
 Chrout
 ---------------------------------------------------------------
 @clib  helper_chrout | Character output function |
        <cfont>int helper_chrout ( char ch, int zero ); <efont> |
 @parm  char  | ch    | Character to print over STDOUT
 @parm  int   | zero  | Zero-Mode<nl>
                        if ch\>;=10 zero is ignored.<nl>
                        if ch\<;10 zero is ignored but the character printed out is ch + 48.<nl>
                        if ch==0 and zero==0 character print is ch + 48.<nl>
                        if ch==0 and zero \<; 0 no character is printed<nl>
                        if ch==0 and zero \>; 0 a space character is printed.<nl>
 @rdesc Value in zero is returned
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../biosints.htm#0x100x0E<linkmid>documentation<linktail>
        for more details.
 @target     SC12  | SC13  | SC11  | SC1x3
 @version    V1.10 | V1.00 | V1.00 | V0.90
*/
int helper_chrout( char ch, int zero )
{
  if( ch < 10 )
  {
    if( ch || ( zero == 0 ))
    {
      zero = 0;
      _AL = '0' + ch;

    }
    else
    {
      if( zero < 0 )
        return zero;

      _AL = ' ';

    } // if

  }
  else
  {
    _AL = ch;
  }

  _AH = 0xE;
  _BH = 0;
  asm int 10h

  return zero;

} // helper_chrout
#endif


#ifdef _helper03_
/*
 @doc DOC_helper03_
 ---------------------------------------------------------------
 Dezout
 ---------------------------------------------------------------
 @clib  helper_dezout | Character output function |
        <cfont>void helper_dezout ( unsigned int x, int zero ); <efont> |
 @parm  int   | x     | Decimal value to print over STDOUT
 @parm  int   | zero  | Zero-Mode<nl>
                        if zero==0, leading zeros are printed as zeros.<nl>
                        if zero\<;0, leading zeros are not printed<nl>
                        if zero\>;0, leading zeros are printed as spaces.<nl>
 @rdesc -- none --
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../biosints.htm#0x100x0E<linkmid>documentation<linktail>
        for more details.
 @target     SC12  | SC13  | SC11  | SC1x3
 @version    V1.10 | V1.00 | V1.00 | V0.90
*/
void helper_dezout( unsigned int x, int zero )
{
  zero = helper_chrout( x / 10000,zero );
  x %= 10000;

  zero = helper_chrout( x / 1000,zero );
  x %= 1000;

  zero = helper_chrout( x / 100,zero );
  x %= 100;

  helper_chrout( x / 10,zero );
  helper_chrout( x % 10,0 );

} // helper_dezout
#endif


#ifdef _helper04_
/*
 @doc DOC_helper04_
 ---------------------------------------------------------------
 Hexdig
 ---------------------------------------------------------------
 @clib  helper_hexdig | Hexadecimal digit output function |
        <cfont>void helper_hexdig ( unsigned char dig ); <efont> |
 @parm  char  | dig  | Hexadecimal digit (nibble) to print over STDOUT
 @rdesc -- none --
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../biosints.htm#0x100x0E<linkmid>documentation<linktail>
        for more details.
 @target     SC12  | SC13  | SC11  | SC1x3
 @version    V1.10 | V1.00 | V1.00 | V0.90
*/
void helper_hexdig( unsigned char dig )
{
  dig &= 0xF;

  if( dig > 9 )
    dig += 'A' - 10;
  else
    dig += '0';

  helper_chrout( dig,-1 );

} // helper_hexdig
#endif


#ifdef _helper05_
/*
 @doc DOC_helper05_
 ---------------------------------------------------------------
 Hexout
 ---------------------------------------------------------------
 @clib  helper_hexout | Hexadecimal integer output function |
        <cfont>void helper_hexout ( unsigned int hex ); <efont> |
 @parm  int  | hex  | Hexadecimal integer to print over STDOUT
 @rdesc -- none --
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../biosints.htm#0x100x0E<linkmid>documentation<linktail>
        for more details.
 @target     SC12  | SC13  | SC11  | SC1x3
 @version    V1.10 | V1.00 | V1.00 | V0.90
*/
void helper_hexout( unsigned hex )
{
  helper_chrout( '0', -1 );
  helper_chrout( 'x', -1 );
  helper_hexdig( hex >> 12 );
  helper_hexdig( hex >>  8 );
  helper_hexdig( hex >>  4 );
  helper_hexdig( hex       );

} // helper_hexout
#endif

/*
 @doc DOC_helper06_
 ---------------------------------------------------------------
 printf
 ---------------------------------------------------------------
 @clib  helper_sprintf | String formatting like the standard 
   library <cfont>sprintf<efont> function. |
        <cfont>int helper_sprintf (char *buffer,<nl>
        <tab><tab><tab><tab><tab>const char *fmt, ... ); <efont> |
 @parm  char *  | buffer  | Output parameter:<s2> Formmatted text
    will be written into this buffer.
 @parm  char *  | fmt  | Format specifier string.<s2> Note: Floating
    point types are not suppported by this implementation.
 @rdesc
   Number of characters output to <cfont>buffer<efont> not counting
   the final NIL string terminating character.<s2> (If @Chip-RTOS version does
   not support this function, then this return value will always be zero.)
 @comm
   Using this function instead of the standard library <cfont>sprintf<efont>
   will reduce a program's memory requirements.<nl>
   <nl>
   <hangpara 6:18>
   <bold>WARNING for small and medium memory models:<bolde><paraend>
   <nl>
   Pointers which are passed to this function after the
    first two declared arguments must be explicitly cast as type "far".<s2>
    (Compiler has no clue about these additional arguments
   so your help is needed.)<s2> For example:<nl>
   <nl>
   <tab><cfont>helper_sprintf(buffer, format, (char far *)string);<efont><nl>
   <nl>
   The exception to this rule are pointers
   for display with the %p format.<s2> These are taken as long by default
   (regardless of your program's memory model) and can be specified explicitly
   as near with the N input format modifier, in which case you should only 
   pass a 16 bit pointer.<s2>  See your compiler's  
   <cfont>sprintf<efont> documentation for more details.<nl>
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../biosints.htm#0xA00x56<linkmid>documentation<linktail>
        for more details.
 @target      SC12  | SC13  | SC11  | SC1x3
 @version     V1.20 | V1.20 | V1.20 | V0.91

 @target_clib V2.03
*/
extern int _Get_Sprintf(void) ;     // Share with  helper_printf()
extern Sprintf_Fptr _sprintf_fptr ;
extern char         _printf_direct ;

#ifdef _helper06_
Sprintf_Fptr _sprintf_fptr ;
char         _printf_direct ;       // Boolean flag

// _Get_Sprintf() returns 0 on success.
int _Get_Sprintf(void)              // Shared with helper_printf()
{
    _BX = 0 ;               // For RTOS version detection.
    _AH = 0x56 ;
    //get pointer of rtos internal sprintf function
    asm{ int 0xA0 } ;       // BIOS SWI
    // BX = 0x1 if vsprintf() suppports direct console output.
    _printf_direct = _BL ;  
    if (_AX==0)
    {
        _sprintf_fptr =  MK_FP(_ES,_DI);
    }
    return _AX ;

}   /* End _Get_Sprintf() */

 
int helper_sprintf(char *buffer, const char *fmt, ...)
{
    int rslt = 0;
    
    if (  (FP_SEG(_sprintf_fptr) != 0)  // Vector set?
        ||(_Get_Sprintf() == 0)         // No vector is set?
       )
    {
        void far *parg;
        va_start(parg, fmt);
        rslt = (*_sprintf_fptr)(buffer, fmt, parg);
        va_end(parg);
    }
    return rslt ;

} // helper_sprintf
#endif

#ifdef _helper06A_
/*
 @doc DOC_helper06A_
 ---------------------------------------------------------------
 printf
 ---------------------------------------------------------------
 @clib  helper_printf | Formatted console output function like the standard 
   library <cfont>printf<efont> function. |
        <cfont>int helper_printf ( const char *fmt, ... ); <efont> |
 @parm  char *  | fmt  | Format specifier string.<s2> Note: Floating
    point types are not suppported by this implementation.
 @rdesc
   Number of characters output to console.<s2> (If @Chip-RTOS version does
   not support this function, then this return value will always be zero.)
 @comm
   Using this function instead of the standard library <cfont>printf<efont>
   will reduce a program's memory requirements.<nl>
   <nl>
   <hangpara 6:18>
   <bold>WARNING for small and medium memory models:<bolde><paraend>
   <nl>
   Pointers which are passed to this function after the
    first declared argument must be explicitly cast as type "far".<s2>
    (Compiler has no clue about these additional arguments
   so your help is needed.)<s2>   For example:<nl>
   <nl>
   <tab><cfont>helper_printf(format, (char far *)string);<efont><nl>
   <nl>
    The exception to this rule are pointers
   for display with the %p format.<s2> These are taken as long by default
   (regardless of your program's memory model) and can be specified explicitly
   as near with the N input format modifier, in which case you should only 
   pass a 16 bit pointer.<s2>  See your compiler's <cfont>printf<efont> 
   documentation for more details.<nl>
   <nl>
   <hangpara 6:18>
   <bold>CAUTION:<bolde><paraend>
   <nl>
   When this function is used with SC1x @Chip-RTOS versions prior to 
   version 1.20 a stack buffer will overflow if the total number of
   characters headed to the console exceeds 179 characters.<s2> For
   subsequent @Chip-RTOS versions there is no such limitation.<nl>

 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../biosints.htm#0xA00x56<linkmid>documentation<linktail>
        for more details.
 @target     SC12  | SC13  | SC11  | SC1x3
 @version    V1.20 | V1.20 | V1.20 | V0.91
*/

///////////////////////////////////////////////////////
// To cover for old RTOS versions.
static int near old_printf(const char * fmt,
                           void far   * parg)
{
  char bfr[180];
  int  rslt = (*_sprintf_fptr)(bfr, fmt, parg);
  helper_strout(bfr);
  return rslt;

} // old_printf()

///////////////////////////////////////////////////////
int helper_printf(const char *fmt, ...)
{
    int rslt = 0;
    
    if (  (FP_SEG(_sprintf_fptr) != 0)  // Vector set earlier?
        ||(_Get_Sprintf() == 0)         // Vector is set now?
       )
    {
        void far *parg ;
        va_start(parg, fmt) ;
        if (_printf_direct == 0)        // Old RTOS that needs stack buffer?
        {
            rslt = old_printf(fmt, parg) ;  // Need caller's stack space.
        }
        else
        {
            // Output directly to console through the RTOS's vsprintf().
            rslt = (*_sprintf_fptr)(MK_FP(0, 0x22), fmt, parg);
        }
        va_end(parg);
    }
    return rslt;

} // helper_printf
#endif


#ifdef _helper07_
/*
 @doc DOC_helper07_
 ---------------------------------------------------------------
 Miscellaneous
 ---------------------------------------------------------------
 @clib  helper_alloc_rtos_mem | Allocate memory from @CHIP-RTOS memory pool (Int21h function 48h) |
        <cfont>unsigned int helper_alloc_rtos_mem(unsigned long size);<efont> |
 @parm  long  | size  | Requested memory size in bytes
 @rdesc If 0, memory allocation failed.
        Else segment of the allocated memory area

 @comm The returned segment can be used to build a normalized pointer to the allocated memory area e.g.
       <cfont>seg=helper_alloc_rtos_mem(1024);<nl>
       ptr = MK_FP(seg,0);<efont>
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../dosemu.htm#0x210x48<linkmid>documentation<linktail>
        for more details.
 @target     SC12  | SC13  | SC11  | SC1x3
 @version    V1.00 | V1.00 | V1.00 | V0.90
*/
unsigned int helper_alloc_rtos_mem(unsigned long size)
{
    // Required paragraph count-> BX.
  _BX = (unsigned int)((size + SEG_MASK) >> SEG_SHIFT) ;
  _AH = 0x48;
  asm int 21h
  return _AX;
}
#endif

#ifdef _helper08_
/*
 @doc DOC_helper08_
 ---------------------------------------------------------------
 Miscellaneous
 ---------------------------------------------------------------
 @clib  helper_release_rtos_mem | Release allocated RTOS memory (Int21h function 49h) |
        <cfont>int helper_release_rtos_mem(unsigned int seg);<efont> |
 @parm  int  | seg  | Segment of the memory area to be released
 @rdesc 0 success<nl>
        -1 failed<nl>
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../dosemu.htm#0x210x49<linkmid>documentation<linktail>
        for more details.
 @target     SC12  | SC13  | SC11  | SC1x3
 @version    V1.00 | V1.00 | V1.00 | V0.90
*/
int helper_release_rtos_mem(unsigned int seg)
{
  unsigned int ax_tmp;

  _ES=seg;
  _AH=0x49;
  asm int 21h

  ax_tmp=_AX;

  if((_FLAGS & 0x0001/*CF*/) && (ax_tmp==9))
  {
     return -1;
  }
  return 0;
}
#endif

#ifdef _helper09_
/*
 @doc DOC_helper09_
 ---------------------------------------------------------------
 Miscellaneous
 ---------------------------------------------------------------
 @clib  helper_alloc_rtos_fast_mem | Allocate memory from @CHIP-RTOS fast memory pool (BIOSIntA0h function 57h) |
        <cfont>unsigned int helper_alloc_rtos_fast_mem(unsigned long size);<efont> |
 @parm  long  | size  | Requested memory size in bytes
 @rdesc If 0, memory allocation failed.
        Else segment of the allocated memory area

 @comm The returned segment can be used to build a normalized pointer to the allocated memory area e.g.
       <cfont>seg=helper_alloc_rtos_fast_mem(1024);<nl>
       ptr = MK_FP(seg,0);<efont>
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../biosints.htm#0xA00x57<linkmid>documentation<linktail>
        for more details.
 @target     SC12  | SC13  | SC11  | SC1x3
 @version    n/a   | n/a   | n/a   | V0.90
*/
unsigned int helper_alloc_rtos_fast_mem(unsigned long size)
{
    // Required paragraph count-> BX.
  _BX = (unsigned int)((size + SEG_MASK) >> SEG_SHIFT) ;
  _AH = 0x57;
  asm int 0A0h
  return _AX;
}
#endif

